@@ -57,12 +57,14 @@ struct _EphyEmbedShellPrivate
 	EphyAdBlockManager *adblock_manager;
 	GtkPageSetup *page_setup;
 	GtkPrintSettings *print_settings;
+	guint object_count;
 	guint single_initialised : 1;
 };
 
 enum
 {
 	PREPARE_CLOSE,
+	QUIT,
 	LAST_SIGNAL
 };
 
@@ -154,7 +156,7 @@ ephy_embed_shell_finalize (GObject *object)
  *
  * Returns the favicons cache.
  *
- * Return value: the favicons cache
+ * Return value: (transfer none): the favicons cache
  **/
 GObject *
 ephy_embed_shell_get_favicon_cache (EphyEmbedShell *shell)
@@ -169,6 +171,12 @@ ephy_embed_shell_get_favicon_cache (EphyEmbedShell *shell)
 	return G_OBJECT (shell->priv->favicon_cache);
 }
 
+/**
+ * ephy_embed_shell_get_global_history:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none):
+ **/
 GObject *
 ephy_embed_shell_get_global_history (EphyEmbedShell *shell)
 {
@@ -182,6 +190,12 @@ ephy_embed_shell_get_global_history (EphyEmbedShell *shell)
 	return G_OBJECT (shell->priv->global_history);
 }
 
+/**
+ * ephy_embed_shell_get_downloader_view:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none):
+ **/
 GObject *
 ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell)
 {
@@ -200,6 +214,12 @@ ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell)
 	return G_OBJECT (shell->priv->downloader_view);
 }
 
+/**
+ * ephy_embed_shell_get_downloader_view_nocreate:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none):
+ **/
 GObject *
 ephy_embed_shell_get_downloader_view_nocreate (EphyEmbedShell *shell)
 {
@@ -252,6 +272,12 @@ impl_get_embed_single (EphyEmbedShell *shell)
 	return G_OBJECT (shell->priv->embed_single);
 }
 
+/**
+ * ephy_embed_shell_get_embed_single:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none):
+ **/
 GObject *
 ephy_embed_shell_get_embed_single (EphyEmbedShell *shell)
 {
@@ -260,6 +286,12 @@ ephy_embed_shell_get_embed_single (EphyEmbedShell *shell)
 	return klass->get_embed_single (shell);
 }
 
+/**
+ * ephy_embed_shell_get_encodings:
+ * @shell: the #EphyEmbedShell
+ *
+ * Return value: (transfer none):
+ **/
 GObject *
 ephy_embed_shell_get_encodings (EphyEmbedShell *shell)
 {
@@ -301,7 +333,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 
 /**
  * EphyEmbed::prepare-close:
- * @shell:
+ * @shell: the #EphyEmbedShell
  * 
  * The ::prepare-close signal is emitted when epiphany is preparing to
  * quit on command from the session manager. You can use it when you need
@@ -315,6 +347,25 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+
+/**
+ * EphyEmbedShell::quit:
+ * @shell: an #EphyEmbedShell
+ * 
+ * The ::quit is emitted when all windows (browser windows, popups,
+ * download windows, etc) are closed and the @shell is ready to be
+ * closed.
+ *
+ * Since: 2.30
+ **/
+	signals[QUIT] =
+		g_signal_new ("quit",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 	
 	g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
@@ -324,7 +375,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
  *
  * Retrieves the default #EphyEmbedShell object
  *
- * ReturnValue: the default #EphyEmbedShell
+ * Return value: (transfer none): the default #EphyEmbedShell
  **/
 EphyEmbedShell *
 ephy_embed_shell_get_default (void)
@@ -338,7 +389,7 @@ ephy_embed_shell_get_default (void)
  *
  * Returns the adblock manager.
  *
- * Return value: the adblock manager
+ * Return value: (transfer none): the adblock manager
  **/
 GObject *
 ephy_embed_shell_get_adblock_manager (EphyEmbedShell *shell)
@@ -383,7 +434,12 @@ ephy_embed_shell_set_page_setup	(EphyEmbedShell *shell,
 	gtk_page_setup_to_file (page_setup, path, NULL);
 	g_free (path);
 }
-		
+
+/**
+ * ephy_embed_shell_get_page_setup:
+ *
+ * Return value: (transfer none):
+ **/
 GtkPageSetup *
 ephy_embed_shell_get_page_setup	(EphyEmbedShell *shell)
 {
@@ -434,6 +490,14 @@ ephy_embed_shell_get_page_setup	(EphyEmbedShell *shell)
 	return priv->page_setup;
 }
 
+/**
+ * ephy_embed_shell_set_print_gettings:
+ * @shell: the #EphyEmbedShell
+ * @settings: the new #GtkPrintSettings object
+ *
+ * Sets the global #GtkPrintSettings object.
+ *
+ **/
 void
 ephy_embed_shell_set_print_settings (EphyEmbedShell *shell,
 				     GtkPrintSettings *settings)
@@ -460,7 +524,15 @@ ephy_embed_shell_set_print_settings (EphyEmbedShell *shell,
 	gtk_print_settings_to_file (settings, path, NULL);
 	g_free (path);
 }
-		
+
+/**
+ * ephy_embed_shell_get_print_settings:
+ * @shell: the #EphyEmbedShell
+ *
+ * Gets the global #GtkPrintSettings object.
+ *
+ * Returns: (transfer none): a #GtkPrintSettings object
+ **/
 GtkPrintSettings *
 ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 {
@@ -478,8 +550,8 @@ ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 		priv->print_settings = gtk_print_settings_new_from_file (path, &error);
 		g_free (path);
 
-		/* Note: the gtk print settings file format is the same as our legacy one,
-		 * so no need to migrate here.
+		/* Note: the gtk print settings file format is the same as our
+		 * legacy one, so no need to migrate here.
 		 */
 
 		if (priv->print_settings == NULL)
@@ -490,3 +562,22 @@ ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 
 	return priv->print_settings;
 }
+
+
+static void
+object_notify_cb (EphyEmbedShell *shell, GObject *object)
+{
+	shell->priv->object_count--;
+	if (shell->priv->object_count == 0)
+		g_signal_emit (shell, signals[QUIT], 0);
+}
+
+void
+_ephy_embed_shell_track_object (EphyEmbedShell *shell, GObject *object)
+{
+	g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
+	g_return_if_fail (G_IS_OBJECT (object));
+
+	g_object_weak_ref (object, (GWeakNotify)object_notify_cb, shell);
+	shell->priv->object_count++;
+}
