@@ -26,7 +26,6 @@
 #include "config.h"
 
 #include "downloader-view.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-adblock-manager.h"
 #include "ephy-debug.h"
 #include "ephy-embed.h"
@@ -40,6 +39,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-history.h"
 #include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-stock-icons.h"
 #include "ephy-string.h"
 #include "ephy-web-view.h"
@@ -49,19 +49,27 @@
 #include <string.h>
 #include <webkit/webkit.h>
 
-static void     ephy_embed_class_init  (EphyEmbedClass *klass);
-static void     ephy_embed_init        (EphyEmbed *gs);
-static void     ephy_embed_constructed (GObject *object);
+static void     ephy_embed_class_init       (EphyEmbedClass *klass);
+static void     ephy_embed_init             (EphyEmbed *gs);
+static void     ephy_embed_constructed      (GObject *object);
+static gboolean ephy_embed_inspect_show_cb  (WebKitWebInspector *inspector,
+                                             EphyEmbed *embed);
+static gboolean ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
+                                             EphyEmbed *embed);
 
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
-struct EphyEmbedPrivate
+struct _EphyEmbedPrivate
 {
   GtkBox *top_widgets_vbox;
   GtkScrolledWindow *scrolled_window;
+  GtkPaned *paned;
   WebKitWebView *web_view;
   EphyHistory *history;
   GtkWidget *inspector_window;
+  GtkWidget *inspector_web_view;
+  GtkWidget *inspector_scrolled_window;
+  gboolean inspector_attached;
   guint is_setting_zoom : 1;
   GSList *destroy_on_transition_list;
 };
@@ -223,6 +231,32 @@ ephy_embed_grab_focus (GtkWidget *widget)
     gtk_widget_grab_focus (child);
 }
 
+static void
+ephy_embed_dispose (GObject *object)
+{
+  EphyEmbed *embed = EPHY_EMBED (object);
+
+  if (embed->priv->inspector_window)
+  {
+    WebKitWebInspector *inspector;
+
+    inspector = webkit_web_view_get_inspector (embed->priv->web_view);
+
+    g_signal_handlers_disconnect_by_func (inspector,
+                                          ephy_embed_inspect_show_cb,
+                                          embed->priv->inspector_window);
+
+    g_signal_handlers_disconnect_by_func (inspector,
+                                          ephy_embed_inspect_close_cb,
+                                          embed->priv->inspector_window);
+
+    gtk_widget_destroy (GTK_WIDGET (embed->priv->inspector_window));
+    embed->priv->inspector_window = NULL;
+  }
+
+  G_OBJECT_CLASS (ephy_embed_parent_class)->dispose (object);
+}
+
 static void
 ephy_embed_finalize (GObject *object)
 {
@@ -251,6 +285,7 @@ ephy_embed_class_init (EphyEmbedClass *klass)
 
   object_class->constructed = ephy_embed_constructed;
   object_class->finalize = ephy_embed_finalize;
+  object_class->dispose = ephy_embed_dispose;
   widget_class->grab_focus = ephy_embed_grab_focus;
 
   g_type_class_add_private (G_OBJECT_CLASS (klass), sizeof(EphyEmbedPrivate));
@@ -258,33 +293,67 @@ ephy_embed_class_init (EphyEmbedClass *klass)
 
 static WebKitWebView *
 ephy_embed_inspect_web_view_cb (WebKitWebInspector *inspector,
-                                  WebKitWebView *web_view,
-                                  gpointer data)
+                                WebKitWebView *web_view,
+                                EphyEmbed *embed)
 {
-  GtkWidget *inspector_sw = GTK_WIDGET (data);
-  GtkWidget *inspector_web_view;
+  return WEBKIT_WEB_VIEW (embed->priv->inspector_web_view);
+}
 
-  inspector_web_view = ephy_web_view_new ();
-  gtk_container_add (GTK_CONTAINER (inspector_sw), inspector_web_view);
+static gboolean
+ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
+                                EphyEmbed *embed)
+{
+  GtkAllocation allocation;
+  gtk_widget_get_allocation (GTK_WIDGET (embed->priv->scrolled_window), &allocation);
 
-  gtk_widget_show_all (gtk_widget_get_toplevel (inspector_sw));
+  embed->priv->inspector_attached = TRUE;
 
-  return WEBKIT_WEB_VIEW (inspector_web_view);
+  /* Set a sane position for the mover */
+  gtk_paned_set_position (embed->priv->paned, allocation.height * 0.5);
+
+  gtk_widget_hide (embed->priv->inspector_window);
+  gtk_widget_reparent (GTK_WIDGET (embed->priv->inspector_scrolled_window),
+                       GTK_WIDGET (embed->priv->paned));
+
+  return TRUE;
+}
+
+static gboolean
+ephy_embed_detach_inspector_cb (WebKitWebInspector *inspector,
+                                EphyEmbed *embed)
+{
+  embed->priv->inspector_attached = FALSE;
+
+  gtk_widget_reparent (GTK_WIDGET (embed->priv->inspector_scrolled_window),
+                       GTK_WIDGET (embed->priv->inspector_window));
+
+  gtk_widget_show_all (embed->priv->inspector_window);
+
+  return TRUE;
 }
 
 static gboolean
 ephy_embed_inspect_show_cb (WebKitWebInspector *inspector,
-                              GtkWidget *widget)
+                            EphyEmbed *embed)
 {
-  gtk_widget_show (widget);
+  if (!embed->priv->inspector_attached) {
+    gtk_widget_show_all (embed->priv->inspector_window);
+    gtk_window_present (GTK_WINDOW (embed->priv->inspector_window));
+  } else
+    gtk_widget_show (embed->priv->inspector_scrolled_window);
+
   return TRUE;
 }
 
 static gboolean
 ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
-                               GtkWidget *widget)
+                             EphyEmbed *embed)
 {
-  gtk_widget_hide (widget);
+  if (!embed->priv->inspector_attached)
+    gtk_widget_hide (embed->priv->inspector_window);
+  else
+    gtk_widget_hide (embed->priv->inspector_scrolled_window);
+
   return TRUE;
 }
 
@@ -335,7 +404,7 @@ request_destination_uri (WebKitWebView *web_view,
   dialog = ephy_file_chooser_new (_("Save"),
                                   window,
                                   GTK_FILE_CHOOSER_ACTION_SAVE,
-                                  CONF_STATE_SAVE_DIR,
+                                  EPHY_PREFS_STATE_SAVE_DIR,
                                   EPHY_FILE_FILTER_ALL_SUPPORTED);
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), suggested_filename);
@@ -713,7 +782,8 @@ download_status_changed_cb (GObject *object,
      */
     g_object_freeze_notify (G_OBJECT (download));
 
-    if (eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS)) {
+    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN,
+                                EPHY_PREFS_AUTO_DOWNLOADS)) {
       perform_auto_download (download);
       /* User won't select a destination, unfreeze. */
       g_object_thaw_notify (G_OBJECT (download));
@@ -747,7 +817,8 @@ download_requested_cb (WebKitWebView *web_view,
                        EphyEmbed *embed)
 {
   /* Is download locked down? */
-  if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK))
+  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK))
     return FALSE;
 
   /* Wait for the request to be sent in all cases, so that we have a
@@ -774,23 +845,30 @@ ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed*)object;
   GtkWidget *scrolled_window;
+  GtkWidget *paned;
   WebKitWebView *web_view;
   WebKitWebInspector *inspector;
-  GtkWidget *inspector_sw;
-
-  embed->priv->top_widgets_vbox = GTK_BOX (gtk_vbox_new (FALSE, 0));
-  gtk_box_pack_start (GTK_BOX (embed), GTK_WIDGET (embed->priv->top_widgets_vbox),
-                      FALSE, FALSE, 0);
-  gtk_widget_show (GTK_WIDGET (embed->priv->top_widgets_vbox));
-
-  scrolled_window = GTK_WIDGET (embed->priv->scrolled_window);
-  gtk_container_add (GTK_CONTAINER (embed), scrolled_window);
-  gtk_widget_show (scrolled_window);
 
+  /* Skeleton */
   web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
+  scrolled_window = GTK_WIDGET (embed->priv->scrolled_window);
+  paned = GTK_WIDGET (embed->priv->paned);
+
   embed->priv->web_view = web_view;
-  gtk_container_add (GTK_CONTAINER (embed->priv->scrolled_window), GTK_WIDGET (web_view));
+
+  gtk_container_add (GTK_CONTAINER (embed->priv->scrolled_window),
+                     GTK_WIDGET (web_view));
+  gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (scrolled_window),
+                   TRUE, FALSE);
+
+  gtk_box_pack_start (GTK_BOX (embed),
+                      GTK_WIDGET (embed->priv->top_widgets_vbox),
+                      FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (embed), paned);
+
+  gtk_widget_show (GTK_WIDGET (embed->priv->top_widgets_vbox));
   gtk_widget_show (GTK_WIDGET (web_view));
+  gtk_widget_show_all (paned);
 
   g_object_connect (web_view,
                     "signal::notify::load-status", G_CALLBACK (load_status_changed_cb), embed,
@@ -799,19 +877,23 @@ ephy_embed_constructed (GObject *object)
                     "signal::notify::zoom-level", G_CALLBACK (zoom_changed_cb), embed,
                     NULL);
 
+  /* The inspector */
+  embed->priv->inspector_web_view  = ephy_web_view_new ();
   embed->priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   inspector = webkit_web_view_get_inspector (web_view);
 
-  inspector_sw = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (inspector_sw),
+  embed->priv->inspector_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (embed->priv->inspector_scrolled_window),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add (GTK_CONTAINER (embed->priv->inspector_window),
-                     inspector_sw);
+                     embed->priv->inspector_scrolled_window);
+  gtk_container_add (GTK_CONTAINER (embed->priv->inspector_scrolled_window),
+                     embed->priv->inspector_web_view);
 
   gtk_window_set_title (GTK_WINDOW (embed->priv->inspector_window),
                         _("Web Inspector"));
   gtk_window_set_default_size (GTK_WINDOW (embed->priv->inspector_window),
-                               600, 400);
+                               800, 600);
 
   g_signal_connect (embed->priv->inspector_window,
                     "delete-event", G_CALLBACK (gtk_widget_hide_on_delete),
@@ -819,11 +901,15 @@ ephy_embed_constructed (GObject *object)
 
   g_object_connect (inspector,
                     "signal::inspect-web-view", G_CALLBACK (ephy_embed_inspect_web_view_cb),
-                    inspector_sw,
+                    embed,
                     "signal::show-window", G_CALLBACK (ephy_embed_inspect_show_cb),
-                    embed->priv->inspector_window,
+                    embed,
                     "signal::close-window", G_CALLBACK (ephy_embed_inspect_close_cb),
-                    embed->priv->inspector_window,
+                    embed,
+                    "signal::attach-window", G_CALLBACK (ephy_embed_attach_inspector_cb),
+                    embed,
+                    "signal::detach-window", G_CALLBACK (ephy_embed_detach_inspector_cb),
+                    embed,
                     NULL);
 
   ephy_embed_prefs_add_embed (embed);
@@ -841,6 +927,8 @@ ephy_embed_init (EphyEmbed *embed)
   embed->priv = EPHY_EMBED_GET_PRIVATE (embed);
 
   embed->priv->scrolled_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
+  embed->priv->paned = GTK_PANED (gtk_vpaned_new ());
+  embed->priv->top_widgets_vbox = GTK_BOX (gtk_vbox_new (FALSE, 0));
 
   gtk_scrolled_window_set_policy (embed->priv->scrolled_window,
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
