@@ -14,9 +14,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-module.c,v 1.12 2006/12/15 14:01:55 chpe Exp $
  */
 
 #include "config.h"
@@ -50,35 +49,7 @@ typedef GType (*EphyModuleRegisterFunc) (GTypeModule *);
 static void ephy_module_init		(EphyModule *action);
 static void ephy_module_class_init	(EphyModuleClass *class);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_module_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo type_info =
-		{
-			sizeof (EphyModuleClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) ephy_module_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (EphyModule),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_module_init,
-		};
-
-		type = g_type_register_static (G_TYPE_TYPE_MODULE,
-					       "EphyModule",
-					       &type_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyModule, ephy_module, G_TYPE_TYPE_MODULE)
 
 static gboolean
 ephy_module_load (GTypeModule *gmodule)
@@ -124,7 +95,7 @@ ephy_module_load (GTypeModule *gmodule)
 
 	if (module->library == NULL)
 	{
-		g_warning (g_module_error());
+		g_warning ("%s", g_module_error());
 
 		return FALSE;
 	}
@@ -133,7 +104,7 @@ ephy_module_load (GTypeModule *gmodule)
 	if (!g_module_symbol (module->library, "register_module",
 			      (void *) &register_func))
 	{
-		g_warning (g_module_error());
+		g_warning ("%s", g_module_error());
 		g_module_close (module->library);
 
 		return FALSE;
@@ -152,6 +123,9 @@ ephy_module_load (GTypeModule *gmodule)
 
 	if (module->type == 0)
 	{
+		g_warning ("Failed to register the GType(s)!");
+		g_module_close (module->library);
+
 		return FALSE;
 	}
 
@@ -212,7 +186,7 @@ ephy_module_finalize (GObject *object)
 
 	g_free (module->path);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_module_parent_class)->finalize (object);
 }
 
 static void
@@ -221,8 +195,6 @@ ephy_module_class_init (EphyModuleClass *class)
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 	GTypeModuleClass *module_class = G_TYPE_MODULE_CLASS (class);
 
-	parent_class = (GObjectClass *) g_type_class_peek_parent (class);
-
 	object_class->finalize = ephy_module_finalize;
 
 	module_class->load = ephy_module_load;
